<?php
declare(strict_types=1);

namespace Catequil\Cli\Container;

use Laminas\ServiceManager\Factory\FactoryInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\ContainerInterface;
use Psr\Container\NotFoundExceptionInterface;
use Symfony\Component\Console\Application;

use function is_string;

/**
 * Class ApplicationFactory
 *
 * @package Catequil\Cli\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-cli
 */
final class ApplicationFactory implements FactoryInterface
{
    /**
     * @param ContainerInterface $container
     * @param $requestedName
     * @param array|null $options
     *
     * @return Application
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, ?array $options = null): Application
    {
        $config = $container->get('config')['cli'] ?? [];

        $application = new Application('catequil', 'dev-master');

        foreach ($config['commands'] ?? [] as $command) {
            if (is_string($command)) {
                $command = $container->get($command);
            }

            $application->add($command);
        }

        if (!empty($config['default_command'])) {
            $application->setDefaultCommand(
                $config['default_command']
            );
        }

        return $application;
    }
}
